from flask import Flask, render_template, request, redirect, url_for
import os
import re
import socket
import struct
import subprocess
import fcntl
import time
import logging

logging.basicConfig(filename='app.log', level=logging.DEBUG)

app = Flask(__name__)

# 获取网卡列表，过滤掉 lo 接口
def get_network_interfaces():
    return [iface for iface in os.listdir('/sys/class/net/') if iface != 'lo']

# 获取网卡的当前 IP 地址
def get_ip_address(ifname):
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        ip = fcntl.ioctl(
            s.fileno(),
            0x8915,  # SIOCGIFADDR
            struct.pack('256s', ifname[:15].encode('utf-8'))
        )
        return socket.inet_ntoa(ip[20:24])
    except IOError:
        return '未分配 IP'

# 读取网卡配置文件内容
def read_interface_config(interface):
    config_path = '/etc/network/interfaces'
    if not os.path.exists(config_path):
        return {}
    
    config = {}
    with open(config_path, 'r') as file:
        for line in file:
            match = re.match(r'(\w+)=(.+)', line)
            if match:
                key, value = match.groups()
                config[key] = value.strip()
    return config

# 写入网卡配置文件
def write_interface_config(interface, config):
    config_path = '/etc/network/interfaces'
    with open(config_path, 'w') as file:
        file.write('auto {}\n'.format(interface))
        file.write('iface {} inet {}\n'.format(interface, config['BOOTPROTO'].lower()))
        if config['BOOTPROTO'].lower() == 'static':
            file.write('    address {}\n'.format(config['IPADDR']))
            file.write('    netmask {}\n'.format(config['NETMASK']))
            file.write('    gateway {}\n'.format(config['GATEWAY']))

# 计算网络地址
def calculate_network(ip, netmask):
    ip_parts = list(map(int, ip.split('.')))
    netmask_parts = list(map(int, netmask.split('.')))
    network_parts = [ip_parts[i] & netmask_parts[i] for i in range(4)]
    return '{}.{}.{}.{}{}/{}'.format(
        network_parts[0],
        network_parts[1],
        network_parts[2],
        network_parts[3],
        netmask_to_cidr(netmask)
    )

# 转换子网掩码为 CIDR
def netmask_to_cidr(netmask):
    return sum([bin(int(x)).count('1') for x in netmask.split('.')])

# 重启指定网卡
def restart_interface(interface):
    try:
        subprocess.run(['ifdown', interface], check=True)
        subprocess.run(['ifup', interface], check=True)
        
        # 等待 15 秒
        time.sleep(15)
    except subprocess.CalledProcessError as e:
        print("重启接口 {} 时出错: {}".format(interface, e))

# 添加策略路由规则
def add_routing_rules(interface, ipaddr, netmask, gateway):
    network = calculate_network(ipaddr, netmask)
    
    # 使用 ip 命令添加路由
    subprocess.run(['ip', 'rule', 'add', 'from', ipaddr, 'table', 'rt_{}'.format(interface)])
    subprocess.run(['ip', 'route', 'add', network, 'dev', interface, 'table', 'rt_{}'.format(interface)])
    subprocess.run(['ip', 'route', 'add', 'default', 'via', gateway, 'dev', interface, 'table', 'rt_{}'.format(interface)])

@app.route('/', methods=['GET', 'POST'])
def index():
    interfaces = get_network_interfaces()
    selected_ip = None

    if request.method == 'POST':
        interface = request.form['interface']
        bootproto = request.form['bootproto']

        config = {}
        config['BOOTPROTO'] = bootproto
        config['ONBOOT'] = 'yes'

        if bootproto == 'static':
            ipaddr = request.form['ipaddr']
            netmask = request.form['netmask']
            gateway = request.form['gateway']
            config['IPADDR'] = ipaddr
            config['NETMASK'] = netmask
            config['GATEWAY'] = gateway
            add_routing_rules(interface, ipaddr, netmask, gateway)
        else:
            config.pop('IPADDR', None)
            config.pop('NETMASK', None)
            config.pop('GATEWAY', None)

        write_interface_config(interface, config)
        restart_interface(interface)

        return redirect(url_for('index'))

    if 'interface' in request.args:
        selected_interface = request.args.get('interface')
        selected_ip = get_ip_address(selected_interface)

    return render_template('index.html', interfaces=interfaces, selected_ip=selected_ip)

if __name__ == '__main__':
    app.run(host='0.0.0.0', port=5000)
